{ ****************************************************************************
    Copyright (c) 2012 by Michael Van Canneyt - C&L verlag

    Android application for managing absentees. Visual part
  ****************************************************************************}

unit absenteeapp;

{$mode objfpc}{$H+}
{$namespace de.freex.absenteeapp}
{$modeswitch unicodestrings}

interface

uses
  AndroidR14, Resources, absenteedata;

type
  TPupilItem = Class
    Text : AWTextView;
    Checkbox : AWCheckbox;
  end;

  { TIDTag }

  TIDTag = Class
    ID : JInt;
    constructor Create(AID : JInt);
  end;

  { TCheckBoxListBinder }

  TCheckBoxListBinder = Class(JLObject,AWSimpleCursorAdapter.InnerViewBinder)
    FListener : AWCompoundButton.InnerOnCheckedChangeListener;
    function setViewValue(view: AVView; cursor: ADCursor; colindex : jint) : jboolean;
  end;

  { TPupilsActivity }

  TPupilsActivity = class(AAListActivity,
                          AWCompoundButton.InnerOnCheckedChangeListener)
  private
    fID: jlong;
  protected
    fDataHelper: TAbsenteeDBHelper;
    fadapter : AWSimpleCursorAdapter;
    fSelectedID: jlong;
    procedure FillItems(Isupdate: Boolean);
  public
    procedure onCreate(savedInstanceState: AOBundle); override;
    procedure onCheckedChanged(cb: AWCompoundButton; checked: jboolean);
  end;

  { TGroupActivity }
  TGroupActivity = class(AAListActivity,
    AWAdapterView.InnerOnItemClickListener)
  private
  protected
    fAdapter: AWSimpleCursorAdapter;
    fDataHelper: TAbsenteeDBHelper;
    fSelectedID: jlong;
    procedure FillItems(isRefresh : Boolean);
  public
    procedure onCreate(savedInstanceState: AOBundle); override;
    procedure onItemClick(aParent: AWAdapterView; aView: AVView;
      aPosition: jint; aID: jlong);
  end;

implementation

{ TCheckBoxListBinder }

function TCheckBoxListBinder.setViewValue(view: AVView; cursor: ADCursor;
  colindex: jint): jboolean;

Var
  cb : AWCheckbox;
  llid,i : jint;
  s : string;

begin
  Result:=(colIndex=2);
  if Result then
    begin
    cb:=AWCheckBox(view);
    llid:=cursor.getint(0);
    cb.setTag(TIDTag.Create(llid));
    i:=cursor.getColumnIndex(TAbsenteeDBHelper.Columnpresencepresence);
    if not Cursor.IsNull(i) then
      begin
      s:=cursor.getstring(i);
      cb.setChecked(s='+');
      end;
    cb.setOnCheckedChangeListener(flistener);
    end;
end;

{ TIDTag }

constructor TIDTag.Create(AID : JInt);
begin
  ID:=AID;
end;


procedure TPupilsActivity.onCheckedChanged(cb: AWCompoundButton;
  checked: jboolean);

Var
  LLID : Integer;

begin
  LLID:=TIDTag(cb.getTag()).ID;
  FDataHelper.Pupilpresence[LLID]:=cb.isChecked;
end;

{ TPupilsActivity }

procedure TPupilsActivity.FillItems(Isupdate: Boolean);

var
  cur: ADCursor;
  colfrom: array[0..1] of JLString;
  b : TCheckBoxListBinder;

begin
  if not IsUpdate then
    begin
    cur:=fDataHelper.GetPupilPresenceFromGroup(FID);
    startManagingCursor(cur);
    colfrom[0] := TAbsenteeDBHelper.columnPupilName;
    colfrom[1] := TAbsenteeDBHelper.Columnpresencepresence;
    fAdapter := AWSimpleCursorAdapter.Create(Self,
                  R.layout.pupil_list_item, cur, colfrom,
                  [R.id.pupil_name,R.id.present]);
    b:=TCheckBoxListBinder.create;
    b.flistener:=Self;
    fadapter.setViewBinder(b);
    setListAdapter(fAdapter);
    end
  else
    fAdapter.getCursor.requery;
end;

procedure TPupilsActivity.onCreate(savedInstanceState: AOBundle);

begin
  inherited onCreate(savedInstanceState);
  fDataHelper := TAbsenteeDBHelper.Create(Self);
  fID := getIntent.getLongExtra('de.freex.absenteeapp.ID', 0);
  if fID = 0 then
    raise EAbsenteeData.Create('No ID for absentee app given');
  setTitle(R.strings.name_pupils);
  UpdateData(False);
end;


{$R *.res}

{.$R *.res}


{ TGroupActivity }

procedure TGroupActivity.FillItems(isRefresh : Boolean);
var
  cur: ADCursor;
  colfrom: array[0..0] of JLString;
begin
  if Not isRefresh then
    begin
    cur := fDataHelper.GetAllGroups;
    startManagingCursor(cur);
    colfrom[0]:=TAbsenteeDBHelper.ColumnGroupName;
    fAdapter:=AWSimpleCursorAdapter.Create(Self,
                R.layout.group_list_item, cur, colfrom,
               [R.id.group_name]);
    setListAdapter(fAdapter);
    end
  else
    fAdapter.getCursor.reQuery;
end;

procedure TGroupActivity.onCreate(savedInstanceState: AOBundle);
begin
  inherited onCreate(savedInstanceState);
  fDataHelper := TAbsenteeDBHelper.Create(Self);
  setTitle(R.strings.name_groups);
  FillItems(False);
  getListView.setOnItemClickListener(Self);
end;

procedure TGroupActivity.onItemClick(aParent: AWAdapterView; aView: AVView;
  aPosition: jint; aID: jlong);
var
  intent: ACIntent;
begin
  if aParent.getContext <> Self then
    Exit;
  intent := ACIntent.Create(Self, JLClass(TPupilsActivity));
  intent.putExtra('de.freex.absenteeapp.ID', aID);
  startActivity(intent);
end;

end.

