{ ****************************************************************************
    Copyright (c) 2012 by Michael Van Canneyt - C&L verlag

    Android application for managing absentees. Database backend.
  ****************************************************************************}

unit absenteedata;

{$namespace de.freex.absenteeapp}
{$mode objfpc}{$H+}
{$modeswitch unicodestrings}

interface

uses
  AndroidR14;

type
  EAbsenteeData = class(JLException)
  end;

  { TAbsenteeDBHelper }

  TAbsenteeDBHelper = class(ADSSqliteOpenHelper)
  strict private
    function GetNameOfGroup(aID: jlong): String;
    procedure SetNameOfGroup(aID: jlong; aValue: String);
    function GetNameOfPupil(aID: jlong): String;
    procedure SetNameOfPupil(aID: jlong; aValue: String);
    function GetPupilpresence(aID: jlong): boolean;
    procedure SetPupilpresence(aID: jlong; aValue: boolean);
  private
    const
      DatabaseVersion = 1;
      DatabaseName = 'absentees';
      CreateTableGroup = 'create table Groups (' +
                               '  gr_id integer primary key autoincrement,' +
                               '  gr_name varchar not null' +
                               ')';
      CreateTablePupil = 'create table Pupil (' +
                             '  pu_id integer primary key autoincrement,' +
                             '  pu_name varchar not null' +
                             ')';
      CreateTablePupilGroup = 'create table PupilGroup (' +
                         '  pg_id integer primary key autoincrement,' +
                         '  pg_Pupil_fk integer not null,' +
                         '  pg_Group_fk integer not null,' +
                         '  foreign key(pg_Pupil_fk) references Pupil (pu_id)' +
                         '    on delete cascade,' +
                         '  foreign key(pg_Group_fk) references Groups (gr_id)' +
                         '    on delete cascade' +
                         ')';
      CreateTablepresence = 'create table presence (' +
                             '  pr_id integer primary key autoincrement,' +
                             '  pr_Pupil_fk id not null,' +
                             '  pr_date date not null default CURRENT_DATE,' +
                             '  pr_code integer, '+
                             '  foreign key(pr_Pupil_fk) references Pupil(pu_id)' +
                             '    on delete cascade' +
                             ')';
  public
    const
      ColumnGroupName = 'gr_name';
      ColumnPupilName = 'pu_name';
      ColumnGroupID = 'gr_id';
      ColumnPupilID = 'pu_id';
      ColumnpresenceID = 'pr_id';
      ColumnpresencePupilfk = 'pr_Pupil_fk';
      Columnpresencepresence = 'pr_code';
      TableGroup = 'Groups';
      TablePupil = 'Pupil';
      Tablepresence = 'presence';

  public
    constructor Create(aContext: ACContext);
    procedure onCreate(aDatabase: ADSSqliteDatabase); override;
    procedure onUpgrade(aDatabase: ADSSqliteDatabase;
      aOldVersion, aNewVersion: jint); override;
    procedure onOpen(aDatabase: ADSSqliteDatabase); override;

    function GetAllGroups: ADCursor;
    function GetPupilsFromGroup(aGroup: jlong): ADCursor;
    function GetPupilpresenceFromGroup(aGroup: jlong): ADCursor;

    procedure DeletePupil(aID: jlong);
    procedure DeleteGroup(aID: jlong);
    procedure Deletepresence(aID: jlong);
  public
    property GroupNaam[aID: jlong]: String read GetNameOfGroup write SetNameOfGroup;
    property PupilNaam[aID: jlong]: String read GetNameOfPupil write SetNameOfPupil;
    property Pupilpresence[aID: jlong]: Boolean read GetPupilpresence  write SetPupilpresence;
  end;

implementation

{ TAbsenteeDBHelper }

constructor TAbsenteeDBHelper.Create(aContext: ACContext);
begin
  inherited Create(aContext, DatabaseName, Nil, DatabaseVersion);
end;

procedure TAbsenteeDBHelper.onCreate(aDatabase: ADSSqliteDatabase);

Const
  LLFirst : Array[1..12] of String =
          ('Andre','Bieke','Celine','Dirk','Elodie','Frouke',
           'Guy','Herman','Ines','Jo','Kurt','Louise');
  LLLast : Array[1..12] of String =
          ('Anthonissen','Bervoets','Courtois','Dedecker','Enfoire','Foudetoi',
           'Guylain','Horsdoeuvre','Ildefonse','Janplezier','Karmelitus','Levoisin');

Var

  I : integer;
  m,n,q : string;

begin
  aDatabase.execSQL(CreateTableGroup);
  aDatabase.execSQL(CreateTablePupil);
  aDatabase.execSQL(CreateTablePupilGroup);
  aDatabase.execSQL(CreateTablepresence);
  // Klassen
  For I:=1 to 6 do
    begin
    n := JLLong.Create(i).toString;
    Q:='insert into Groups (gr_name) values (''Group '+n+'A'')';
    aDatabase.ExecSQL(Q);
    Q:='insert into Groups (gr_name) values (''Group '+n+'B'')';
    aDatabase.ExecSQL(Q);
    end;
  // Pupils
  For I:=1 to 12 do
    begin
    Q:='insert into Pupil (pu_name) values ('''+llFirst[i]+' '+LLLast[i]+''')';
    aDatabase.ExecSQL(Q);
    Q:='insert into Pupil (pu_name) values ('''+llFirst[13-i]+' '+LLLast[i]+''')';
    aDatabase.ExecSQL(Q);
    end;
  // Put pupils in groups
  For I:=1 to 24 do
    begin
    n := JLLong.Create(i).toString;
    m := JLLong.Create(i div 2).toString;
    Q:='insert into PupilGroup (pg_Pupil_fk, pg_Group_fk) values ('+n+' , '+m+')';
    aDatabase.ExecSQL(Q);
    end;
end;

procedure TAbsenteeDBHelper.onUpgrade(aDatabase: ADSSqliteDatabase;
  aOldVersion, aNewVersion: jint);
begin
  // nothing
end;

procedure TAbsenteeDBHelper.onOpen(aDatabase: ADSSqliteDatabase);
begin
  inherited onOpen(aDatabase);
  if not aDatabase.isReadOnly then
    aDatabase.ExecSQL('pragma foreign_keys=on;');
end;

function TAbsenteeDBHelper.GetPupilsFromGroup(aGroup: jlong): ADCursor;

var
  query: String;

begin
  query := JLLong.Create(aGroup).toString;
  query := 'select pu_id as _id, pu_name'+
           ' from '+
           ' PupilGroup left join Pupil on (pg_Pupil_fk=pu_id)'+
           ' where (pg_Group_fk='+Query+') order by pu_name asc';
  Result := getWritableDatabase.rawQuery(query,nil);
end;

function TAbsenteeDBHelper.GetPupilpresenceFromGroup(aGroup: jlong): ADCursor;

var
  query: String;

begin
  query := JLLong.Create(aGroup).toString;
  query := 'select pu_id as _id, pu_name, pr_code' +
           ' from ' +
           ' PupilGroup left join Pupil on (pg_Pupil_fk=pu_id)' +
           ' left join presence on ((pr_Pupil_fk=pu_id) and (pr_date=date(''now''))) '+
           ' where (pg_Group_fk='+Query+') order by pu_name asc';
  Result := getWritableDatabase.rawQuery(query,nil);
end;

function TAbsenteeDBHelper.GetAllGroups: ADCursor;
var
  query: String;
begin
  query := 'select gr_id as _id, gr_name from Groups order by gr_name asc';
  Result := getWritableDatabase.rawQuery(query, Nil);
end;



function TAbsenteeDBHelper.GetNameOfPupil(aID: jlong): String;
var
  cur: ADCursor;
  id: String;
  query: JLString;
begin
  id := JLLong.Create(aID).toString;
  cur := getWritableDatabase.rawQuery(
           'select pu_name from Pupil where pu_id = ' + id, Nil);
  if cur.getCount = 0 then
    raise EAbsenteeData.Create('Pupil niet gevonden');
  cur.moveToFirst;
  Result := cur.getString(0);
end;

function TAbsenteeDBHelper.GetPupilpresence(aID: jlong): Boolean;
var
  cur: ADCursor;
  id, sql: String;
  query: JLString;

begin
  id := JLLong.Create(aID).toString;
  sql := 'select pr_code from presence where (pr_pupil_fk='+id+')'+
         'and (pr_date=date(''now''))';
  cur := getWritableDatabase.rawQuery(sql,nil);
  if cur.getCount = 0 then
    Result:=False
  else
    begin
    cur.moveToFirst;
    sql:='+';
    Result := (cur.getString(0)=sql);
    end;
end;

procedure TAbsenteeDBHelper.SetPupilpresence(aID: jlong; aValue: Boolean);
var
  where, l: String;
  values: ACContentValues;
  cur: ADCursor;
  id, sql: String;
  query: JLString;
  db : ADSSQLiteDatabase;

begin
  id := JLLong.Create(aID).toString;
  db:=getWritableDatabase;
  sql := 'select pr_id from presence where (pr_pupil_fk='+id+')'+
         'and (pr_date=date(''now''))';
  AULog.d('absentee',sql);
  cur := db.rawQuery(sql,nil);
  if (cur.getCount=0) then
    begin
    sql:='insert into presence (pr_pupil_fk, pr_date, pr_code) '+
         ' values ('+id+',date(''now''),';
    if aValue then
      sql:=sql+'''+'')'
    else
      sql:=sql+'''-'')';
    AULog.d('absentee','inserting : '+sql);
    db.execSQL(sql);
    end
  else
    begin
    cur.moveToFirst;
    id := JLLong.create(cur.getint(0)).tostring;
    cur.close;
    sql:='update presence set pr_code=';
    if aValue then
      sql:=sql+'''+'''
    else
      sql:=sql+'''-''';
    sql:=sql+' where (pr_id = '+id+')';
    AULog.d('absentee','Updating : '+sql);
    db.execsql(sql);
    end;
end;

procedure TAbsenteeDBHelper.SetNameOfGroup(aID: jlong; aValue: String);
var
  where: String;
  values: ACContentValues;
begin
  where := JLLong.Create(aID).toString;
  where := 'gr_id = ' + where;
  values := ACContentValues.create;
  values.put(ColumnGroupName, aValue);
  getWritableDatabase.update(TableGroup, values, where, Nil);
end;

function TAbsenteeDBHelper.GetNameOfGroup(aID: jlong): String;
var
  cur: ADCursor;
  id: String;
  query: JLString;
begin
  id := JLLong.Create(aID).toString;
  cur := getWritableDatabase.rawQuery(
           'select gr_name from Groups where gr_id = ' + id, Nil);
  if cur.getCount = 0 then
    raise EAbsenteeData.Create('Group not found');
  cur.moveToFirst;
  Result := cur.getString(0);
end;

procedure TAbsenteeDBHelper.SetNameOfPupil(aID: jlong; aValue: String);
var
  where: String;
  values: ACContentValues;
begin
  where := JLLong.Create(aID).toString;
  where := 'pu_id = ' + where;
  values := ACContentValues.create;
  values.put(ColumnPupilName, aValue);
  getWritableDatabase.update(TablePupil, values, where, Nil);
end;

procedure TAbsenteeDBHelper.DeleteGroup(aID: jlong);
var
  where: String;
begin
  where := JLLong.Create(aID).toString;
  where := 'gr_id = ' + where;
  getWritableDatabase.delete(TableGroup, where, Nil);
end;

procedure TAbsenteeDBHelper.DeletePupil(aID: jlong);
var
  where: String;
begin
  where := JLLong.Create(aID).toString;
  where := 'pu_id = ' + where;
  getWritableDatabase.delete(TablePupil, where, Nil);
end;

procedure TAbsenteeDBHelper.Deletepresence(aID: jlong);
var
  where: String;
begin
  where := JLLong.Create(aID).toString;
  where := '_id = ' + where;
  getWritableDatabase.delete(Tablepresence, where, Nil);
end;

end.

